/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.ScriptingBean;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

import org.apache.bsf.BSFException;
import org.apache.bsf.BSFManager;

import ca.uvic.csr.shrimp.ShrimpConstants;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpApplication;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.AbstractShrimpTool;
import ca.uvic.csr.shrimp.gui.ActionManager.ActionManager;
import ca.uvic.csr.shrimp.gui.ActionManager.CheckBoxAction;
import ca.uvic.csr.shrimp.resource.ResourceHandler;
import ca.uvic.csr.shrimp.util.ShowInBrowserAction;
import ca.uvic.csr.shrimp.util.ShrimpFileFilter;

/**
 *
 * Created on July 2, 2003, 3:24 PM
 * @author  Nasir Rather, Chris Callendar
 */
public class ScriptingTool extends AbstractShrimpTool {
	
	public final static String LAST_SCRIPT_OPEN_DIRECTORY_PROP_KEY = "KEY_LAST_OPEN_DIR";
	public final static String LAST_SCRIPT_SAVE_DIRECTORY_PROP_KEY = "KEY_LAST_SAVE_DIR";
	
	private JPanel mainPanel;
    private JRootPane rootPane;
	private JLabel statusLabel;
	private JEditorPane editorPane;
	private JCheckBoxMenuItem javaScriptMenuItem;
	
	private ScriptingBean scriptingBean;	
	private File scriptFile;

	private Action open;
	private Action save;
	private Action saveAs;
	private Action run;
	private Action close;
	private CheckBoxAction javascript;
	private Action help;
	private Action addTryCatch;

	/** Creates new form ScriptingTool */
	public ScriptingTool(ShrimpProject project) {
		super(ShrimpProject.SCRIPTING_TOOL, project);
		rootPane = new JRootPane();
		mainPanel = new JPanel();
		//rootPane.setLayout(new BorderLayout());
		createActions();
		scriptFile = new File("");
	}
	
	private void initComponents() {
		rootPane.getContentPane().removeAll();
		rootPane.getContentPane().setLayout(new BorderLayout());
		rootPane.getContentPane().add(mainPanel, BorderLayout.CENTER);
		mainPanel.removeAll();
		
		boolean enabled = (project != null);
		open.setEnabled(enabled);
		save.setEnabled(enabled);
		saveAs.setEnabled(enabled);
		run.setEnabled(enabled);
		javascript.setEnabled(enabled);
		close.setEnabled(true);

		if (project == null) {
			mainPanel.setLayout(new BorderLayout());
			mainPanel.add(new JLabel("No Project."), BorderLayout.CENTER);
			return;	
		}
		
		rootPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke("F5"), "runScript");
		rootPane.getActionMap().put("runScript", run);
		
		statusLabel = new JLabel();
		javaScriptMenuItem = new JCheckBoxMenuItem();
		javaScriptMenuItem.setSelected(true);
		
		mainPanel.setLayout(new GridBagLayout());

//		JToolBar toolbar = new JToolBar();
//		toolbar.setBorder(new EtchedBorder());
//		toolbar.setFloatable(false);
//		JButton runButton = new JButton();
//		runButton.setText("Run");
//		runButton.setToolTipText("Run (F5)");
//		runButton.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent evt) {
//				runActionPerformed(evt);
//			}
//		});
//		toolbar.add(runButton);
//
//		JButton stepButton = new JButton();
//		stepButton.setText("Step");
//		stepButton.setToolTipText("Coming soon.");
//		stepButton.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent evt) {
//				stepActionPerformed(evt);
//			}
//		});
//		stepButton.setEnabled(false);
//		toolbar.add(stepButton);
//		rootPane.getContentPane().add(toolbar, BorderLayout.NORTH);
		
		editorPane = new JEditorPane();
		editorPane.setPreferredSize(new Dimension(450, 300));
		editorPane.getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, KeyEvent.CTRL_DOWN_MASK), "runScript2");
		editorPane.getActionMap().put("runScript2", run);
		
		// example code
		StringBuffer code = new StringBuffer();
		code.append("constants = bsf.lookupBean(\"scriptingConstants\");\n");
		code.append("mainView = bsf.lookupBean(\"mainViewScriptingBean\");\n\n");
		int position = code.length(); 
		editorPane.setText(code.toString());
		editorPane.setCaretPosition(position);
		
		JScrollPane scrollPane = new JScrollPane(editorPane);
		scrollPane.setBorder(new CompoundBorder(new EmptyBorder(new Insets(2, 2, 2, 2)), new EtchedBorder()));

		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 1.0;
		mainPanel.add(scrollPane, gridBagConstraints);

		statusLabel.setBackground(new Color(204, 204, 204));
		statusLabel.setText(">");
		statusLabel.setBorder(new EtchedBorder());
		gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.gridx = 0;
		gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
		gridBagConstraints.weighty = 0.0;
		mainPanel.add(statusLabel, gridBagConstraints);
		
		recreateMenuBar();

	}

	
	private void createActions() {
		actionManager = new ActionManager();

		open = new AbstractAction("Open...") {
			public void actionPerformed(ActionEvent evt) {
				loadActionPerformed(evt);
			}
		};
		actionManager.addAction(open, ShrimpConstants.MENU_FILE, ShrimpConstants.MENU_FILE, 1);
		
		save = new AbstractAction("Save") {
			public void actionPerformed(ActionEvent evt) {
				saveActionPerformed(evt);
			}
		};
		actionManager.addAction(save, ShrimpConstants.MENU_FILE, ShrimpConstants.MENU_FILE, 2);
		
		saveAs = new AbstractAction("Save as...") {
			public void actionPerformed(ActionEvent evt) {
				saveAsActionPerformed(evt);
			}
		};
		actionManager.addAction(saveAs, ShrimpConstants.MENU_FILE, ShrimpConstants.MENU_FILE, 3);
		
		close = new AbstractAction("Close") {
			public void actionPerformed(ActionEvent evt) {
				closeActionPerformed(evt);
			}
		};
		actionManager.addAction(close, ShrimpConstants.MENU_FILE, ShrimpConstants.MENU_FILE, 4);
		
		run = new AbstractAction("Run (F5)") {
			public void actionPerformed(ActionEvent evt) {
				runActionPerformed(evt);
			}
		};
		actionManager.addAction(run, "Script", "Script", 1);

		addTryCatch = new AbstractAction("Add a try catch block") {
			public void actionPerformed(ActionEvent evt) {
				addTryCatchActionPerformed(evt);
			}
		};
		actionManager.addAction(addTryCatch, "Script", "Script", 2);
		
		javascript = new CheckBoxAction("Javascript") {
			public void startAction() {
				scriptingBean.setLanguage("javascript");
			}
		};
		javascript.setChecked(true);
		actionManager.addAction(javascript, "Language", "Language", 1);
		
		URL url = null; //ResourceHandler.getFileURL("doc/scripting/index.html");
	    try {
	        // use the online version
            url = new URL ("http://www.cs.uvic.ca/~chisel/projects/shrimp/scripting-api/ca/uvic/csr/shrimp/ScriptingBean/MainViewScriptingBean.html");
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
		if (url != null) {
			help = new ShowInBrowserAction("Scripting API", url.toString(), ResourceHandler.getIcon("icon_help.gif"));
			actionManager.addAction(help, "Scripting API", "Group-A", 1);
		}
	}
	

	private void loadActionPerformed(ActionEvent evt) {
		ShrimpApplication app = ApplicationAccessor.getApplication();
		JFileChooser chooser = new JFileChooser();
		ShrimpFileFilter jsFilter = new ShrimpFileFilter("js", "Javascript");
		chooser.addChoosableFileFilter(jsFilter);
		String lastDirectory = app.getProperties().getProperty(LAST_SCRIPT_OPEN_DIRECTORY_PROP_KEY);
		if (lastDirectory != null) {
			chooser.setCurrentDirectory(new File(lastDirectory));
		} else {
			chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
		}
		chooser.setDialogType(JFileChooser.OPEN_DIALOG);
		int state = chooser.showOpenDialog(app.getParentFrame());
		
		if (state == JFileChooser.APPROVE_OPTION) {
			scriptFile = chooser.getSelectedFile();
			app.getProperties().setProperty(LAST_SCRIPT_OPEN_DIRECTORY_PROP_KEY, scriptFile.getParent());
			try {
				scriptingBean.setLanguage(BSFManager.getLangFromFilename(scriptFile.getName()));				
				BufferedReader reader = new BufferedReader(new FileReader(scriptFile));
				editorPane.read(reader, new PlainDocument());
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (BSFException e1) {
				e1.printStackTrace();
			}			
		}
	}

	private void saveAsActionPerformed(ActionEvent evt) {
		ShrimpApplication app = ApplicationAccessor.getApplication();
		JFileChooser chooser = new JFileChooser();
		ShrimpFileFilter filter = null;
		
		if(scriptingBean.getLanguage().equals("javascript")) {
			filter = new ShrimpFileFilter("js", "Javascript");
		}
		
		chooser.addChoosableFileFilter(filter);
		String lastDirectory = app.getProperties().getProperty(LAST_SCRIPT_SAVE_DIRECTORY_PROP_KEY);
		if (lastDirectory != null) {
			chooser.setCurrentDirectory(new File(lastDirectory));
		} else {
			chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
		}
		chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		int state = chooser.showSaveDialog(app.getParentFrame());
			
		if (state == JFileChooser.APPROVE_OPTION) {
			scriptFile = chooser.getSelectedFile();
			app.getProperties().setProperty(LAST_SCRIPT_SAVE_DIRECTORY_PROP_KEY, scriptFile.getParent());
			if (!scriptFile.getName().toLowerCase().endsWith(".js")) {
				scriptFile = new File (scriptFile.getAbsolutePath() + ".js");
			}
			try {
				editorPane.write(new FileWriter(scriptFile));
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private void saveActionPerformed(ActionEvent evt) {
		if (scriptFile.exists()) {
			try {
				editorPane.write(new FileWriter(scriptFile));
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		else {
			saveAsActionPerformed(evt);
		}
	}

	protected void stepActionPerformed(ActionEvent evt) {
		// Add your handling code here:
	}
	
	private void setStatusText(String text) {
		setStatusText(text, false);
	}
	
	private void setStatusText(String text, boolean error) {
		if (statusLabel != null) {
			statusLabel.setText(text);
			statusLabel.setForeground((error ? Color.red : Color.black));
		}
	}

	private void runActionPerformed(ActionEvent evt) {
		setStatusText("> Running Script...");
		runScript(editorPane.getText());
	}
	
	private boolean runScript(String scriptAsString) {
		try {	
			scriptingBean.runScriptString(scriptAsString);		
			setStatusText("> Script finished");
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			setStatusText("> Script error: " + e.getMessage(), true);
		} 
		return false;
    }
	
	private void closeActionPerformed(ActionEvent e) {
		if (rootPane != null) {
			// find the parent window
			Container container = rootPane.getParent();
			while (container != null) {
				if (container instanceof Window) {
					((Window)container).dispose();
					break;
				}
				container = container.getParent();
			}
		}		
	}
	
	/**
	 * Inserts a try catch block at the cursor position. 
	 * @param e
	 */
	private void addTryCatchActionPerformed(ActionEvent e) {
		if (editorPane ==  null)
			return;
		
		StringBuffer buffer = new StringBuffer();
		int end = editorPane.getCaretPosition();	// editorPane.getText().length();
		buffer.append("\ntry {\n    ");
		int position = end + buffer.length(); 
		buffer.append("\n} catch (e) {\n    mainView.alert(\"Caught exception: \"+e);\n}");
		try {
			editorPane.getDocument().insertString(end, buffer.toString(), null);
			editorPane.setCaretPosition(position);
		} catch (BadLocationException ignore) {
		}		
	}
    
	public Component getGUI() {
		return rootPane;
	}
	
	public void disposeTool() {
	}


	public void refresh() {
		String text = editorPane != null ? editorPane.getText() : null; // save existing text when refreshing
		if (project != null) {
			try {
				scriptingBean = (ScriptingBean) project.getBean(ShrimpProject.SCRIPTING_BEAN);
			} catch (BeanNotFoundException e) {
				e.printStackTrace();
			}
		}

		initComponents();
		if (text != null) {
			editorPane.setText(text);
		}
	}

    /* (non-Javadoc)
     * @see ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool#clear()
     */
    public void clear() {
        // nothing for now
    }
	
}
