/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.adapter;

import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JLabel;

import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.DisplayBean.event.ShrimpKeyEvent;
import ca.uvic.csr.shrimp.DisplayBean.event.ShrimpMouseAndKeyAdapter;
import ca.uvic.csr.shrimp.DisplayBean.event.ShrimpMouseEvent;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.usercontrols.UserAction;
import ca.uvic.csr.shrimp.usercontrols.UserEvent;
 
/**
 * This adapter takes mouse and keyboard events, and maps them to the
 * appropriate user action.
 *
 * @author Casey Best, Chris Callendar
 * @date Jan 30, 2001
 */ 
public class ShrimpInputAdapter extends ShrimpMouseAndKeyAdapter {
	
	private Vector mousePressedEvents;
	private Vector mouseReleasedEvents;
	private Vector mouseWheelEvents;
	private Vector keyPressedEvents;
	private Vector keyReleasedEvents;
	
	private boolean isCtrlPressed;
	private boolean isShiftPressed;
	private boolean isAltPressed;
	
	private SelectorBean selectorBean;
	private ShrimpTool tool;
	
	private JLabel inputLabel;

	public ShrimpInputAdapter(ShrimpTool tool, JLabel inputLabel) {
		this(tool);
		this.inputLabel = inputLabel;
	}
	
	public ShrimpInputAdapter(ShrimpTool tool) {
		this.tool = tool;
	
		mousePressedEvents = new Vector();
		mouseReleasedEvents = new Vector();
		mouseWheelEvents = new Vector();
		keyPressedEvents = new Vector();
		keyReleasedEvents = new Vector();

		isCtrlPressed = false;
		isShiftPressed = false;
		isAltPressed = false;
	}

	/**
	 * If the action must start and then stop this method will call {@link UserAction#stopAction()}, otherwise
	 * it will call {@link UserAction#startAction()}.
	 * @param action
	 */
//	private void startOrStopAction(UserAction action) {
//		if (action.mustStartAndStop()) {
//			stopAction(action);
//		} else { 
//			startAction(action);
//		}
//	}

	/**
	 * Starts the {@link UserAction} and updates the action label.
	 */
	private void startAction(UserAction action) {
		if (action.canStart()) {
			if (inputLabel != null) {
				final String txt = "Action: " + action.getActionName() + " ";
				inputLabel.setText(txt);
				inputLabel.setToolTipText(txt);
			}
			//System.out.println("Starting: " + action.getActionName());
			action.startAction();
		} else {
			//System.err.println("Couldn't start action: " + action.getActionName());
		}
	}
	
	/**
	 * Stops the {@link UserAction} and clears the action label.
	 */
	private void stopAction(UserAction action) {
		if (inputLabel != null) {
			inputLabel.setText("");
			inputLabel.setToolTipText("");
		}
		//System.out.println("Stopping: " + action.getActionName());
		action.stopAction();
	}
	
	/** 
	 * @see ca.uvic.csr.shrimp.DisplayBean.event.ShrimpMouseListener#mousePressed(ca.uvic.csr.shrimp.DisplayBean.event.ShrimpMouseEvent)
	 */
	public void mousePressed(ShrimpMouseEvent e) {
		if (!handleAnyMouseEvent(e)) {
			return;
		}
			
		// check the left mouse button
		if (e.isLeftMouseButton()) {
			for (int i = 0; i < mousePressedEvents.size(); i++) {
				UserEvent userEvent = (UserEvent) mousePressedEvents.elementAt(i);
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					if (userEvent.getKeyOrButton() == UserEvent.LEFT_MOUSE_BUTTON && e.getClickCount() == 1) {
						startAction(userEvent.getAction());
					} else if (userEvent.getKeyOrButton() == UserEvent.DOUBLE_CLICK__LEFT_MOUSE_BUTTON && e.getClickCount() >= 2) {
						startAction(userEvent.getAction());
					}
				}
			}
			
		// check the middle mouse button
		} else if (e.isMiddleMouseButton()) {
			for (int i = 0; i < mousePressedEvents.size(); i++) {
				UserEvent userEvent = (UserEvent) mousePressedEvents.elementAt(i);
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					if (userEvent.getKeyOrButton() == UserEvent.MIDDLE_MOUSE_BUTTON && e.getClickCount() == 1) {
						startAction(userEvent.getAction());
					} else if (userEvent.getKeyOrButton() == UserEvent.DOUBLE_CLICK__MIDDLE_MOUSE_BUTTON && e.getClickCount() >= 2) {
						startAction(userEvent.getAction());
					}
				}
			}
			
		// check the right mouse button
		} else if (e.isRightMouseButton()) {
			for (int i = 0; i < mousePressedEvents.size(); i++) {
				UserEvent userEvent = (UserEvent) mousePressedEvents.elementAt(i);
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					if (userEvent.getKeyOrButton() == UserEvent.RIGHT_MOUSE_BUTTON && e.getClickCount() == 1) {
						startAction(userEvent.getAction());
					} else if (userEvent.getKeyOrButton() == UserEvent.DOUBLE_CLICK__RIGHT_MOUSE_BUTTON && e.getClickCount() >= 2) {
						startAction(userEvent.getAction());
					}
				}
			}
		}
	}

	/** 
	 * @see ca.uvic.csr.shrimp.DisplayBean.event.ShrimpMouseListener#mouseReleased(ca.uvic.csr.shrimp.DisplayBean.event.ShrimpMouseEvent)
	 */
	public void mouseReleased(ShrimpMouseEvent e) {
		if (!handleAnyMouseEvent(e)) {
			return;
		}
		
		// check the left mouse button
		if (e.isLeftMouseButton()) {
			for (int i = 0; i < mouseReleasedEvents.size(); i++) {
				UserEvent userEvent = (UserEvent) mouseReleasedEvents.elementAt(i);
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					if (userEvent.getKeyOrButton() == UserEvent.LEFT_MOUSE_BUTTON) {
						stopAction(userEvent.getAction());
					}
				}
			}

		// check the middle mouse button
		} else if (e.isMiddleMouseButton()) {
			for (int i = 0; i < mouseReleasedEvents.size(); i++) {
				UserEvent userEvent = (UserEvent) mouseReleasedEvents.elementAt(i);
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					if (userEvent.getKeyOrButton() == UserEvent.MIDDLE_MOUSE_BUTTON) {
						stopAction(userEvent.getAction());
					}
				}
			}
		// check the right mouse button
		} else if (e.isRightMouseButton()) {
			for (int i = 0; i < mouseReleasedEvents.size(); i++) {
				UserEvent userEvent = (UserEvent) mouseReleasedEvents.elementAt(i);
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					if (userEvent.getKeyOrButton() == UserEvent.RIGHT_MOUSE_BUTTON) {
						stopAction(userEvent.getAction());
					}
				}
			}
		}
	}
	
	public void mouseWheelMoved(ShrimpMouseEvent e) {
		// @tag Shrimp.MouseWheel
		if (handleAnyMouseEvent(e)) {
			for (Iterator iter = mouseWheelEvents.iterator(); iter.hasNext(); ) {
				UserEvent userEvent = (UserEvent) iter.next();
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					if (e.isUpWheelRotation() && (userEvent.getKeyOrButton() == UserEvent.MOUSE_WHEEL_UP)) {
						if (userEvent.getAction().mustStartAndStop()) {
							startAction(userEvent.getAction());
							stopAction(userEvent.getAction());
						} else {
							startAction(userEvent.getAction());
						}
					} else 	if (e.isDownWheelRotation() && (userEvent.getKeyOrButton() == UserEvent.MOUSE_WHEEL_DOWN)) {
						if (userEvent.getAction().mustStartAndStop()) {
							startAction(userEvent.getAction());
							stopAction(userEvent.getAction());
						} else {
							startAction(userEvent.getAction());
						}
					}
				}
			}
		}
	}

	/** This method handles all of the mouse events first */
	public boolean handleAnyMouseEvent(ShrimpMouseEvent e) {
		if (tool == null) {
			return false;
		}
		if (inputLabel != null) {
			inputLabel.setText("");
			inputLabel.setToolTipText("");
		}
		
		try {				
			selectorBean = (SelectorBean) tool.getBean(ShrimpTool.SELECTOR_BEAN);
		} catch (BeanNotFoundException bnfe) {
		  	bnfe.printStackTrace();
		  	return false;
		}		
			
		// in case the system didn't throw all the events, this clears the modifiers for us
		if (e.getModifiers() == 0) {
			isCtrlPressed = false;
			isShiftPressed = false;
			isAltPressed = false;
		}
	
		// set the current target
		Object targetObj = e.getTarget();
		if (targetObj != null) {
			selectorBean.setSelected(SelectorBeanConstants.TARGET_OBJECT, targetObj);
		} else  {
			selectorBean.clearSelected(SelectorBeanConstants.TARGET_OBJECT);
		}
		
		// set the current mouse coordinates
		Vector coords = new Vector();
	    coords.addElement(new Double(e.getX()));
	    coords.addElement(new Double(e.getY()));
		selectorBean.setSelected(SelectorBeanConstants.MOUSE_COORDINATES, coords);
		
		return true;
	}
	
    /** Handles all the actions when a key is pressed */
    public void keyPressed(ShrimpKeyEvent e) {
    	int code = e.getKeyCode();
		// in case the system didn't throw all the events, this clears the modifiers for us
		if (e.getModifiers() == 0) {
			isCtrlPressed = false;
			isShiftPressed = false;
			isAltPressed = false;
		}
		if (code == KeyEvent.VK_CONTROL) {
			isCtrlPressed = true;
		} else if (code == KeyEvent.VK_SHIFT) {
			isShiftPressed = true;
		} else if (code == KeyEvent.VK_ALT) {
			isAltPressed = true;
		}

		if (inputLabel != null) {
			inputLabel.setText("");
			inputLabel.setToolTipText("");
		}

    	for (int i = 0; i < keyPressedEvents.size(); i++) {
			UserEvent userEvent = (UserEvent) keyPressedEvents.elementAt(i);
    		if (userEvent.getKeyOrButton() == code) {
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					startAction(userEvent.getAction());
				}
    		}
    	}

    }
    
    /** Handles all the actions when a key is released */
    public void keyReleased(ShrimpKeyEvent e) {
    	int code = e.getKeyCode();
		if (code == KeyEvent.VK_CONTROL) {
			isCtrlPressed = false;
		} else if (code == KeyEvent.VK_SHIFT) {
			isShiftPressed = false;
		} else if (code == KeyEvent.VK_ALT) {
			isAltPressed = false;
		}
		
    	for (int i = 0; i < keyReleasedEvents.size(); i++) {
			UserEvent userEvent = (UserEvent) keyReleasedEvents.elementAt(i);
    		if (userEvent.getKeyOrButton() == code) {
				if ((isShiftPressed == userEvent.isShiftRequired()) && 
					(isCtrlPressed == userEvent.isControlRequired()) && 
					(isAltPressed == userEvent.isAltRequired())) {
					stopAction(userEvent.getAction());
				}
	    	}
    	}
    }

	public void addUserAction(UserAction action) {
		Vector userEvents = action.getUserEvents();
		for (Iterator iterator = userEvents.iterator(); iterator.hasNext();) {
			UserEvent userEvent = (UserEvent) iterator.next();			
			if (action.mustStartAndStop()) {
				if (userEvent.isMouseActivated()) {
					if (userEvent.isMouseWheelEvent()) {
						mouseWheelEvents.addElement(userEvent);
					} else {
						mousePressedEvents.addElement(userEvent);
						mouseReleasedEvents.addElement(userEvent);
					}
				} else {
					keyPressedEvents.addElement(userEvent);
					keyReleasedEvents.addElement(userEvent);
				}
			} else {
				if (userEvent.isMouseActivated()){
					if (userEvent.isMouseWheelEvent()) {
						mouseWheelEvents.addElement(userEvent);
					} else {
						mousePressedEvents.addElement(userEvent);
					}
				} else {
					keyPressedEvents.addElement(userEvent);
				}
			}
		}
	}
	
	public void removeUserAction(UserAction action) {
		Vector userEvents = action.getUserEvents();
		for (Iterator iterator = userEvents.iterator(); iterator.hasNext();) {
			UserEvent userEvent = (UserEvent) iterator.next();			
			if (action.mustStartAndStop()) {
				if (userEvent.isMouseActivated()) {
					if (userEvent.isMouseWheelEvent()) {
						mouseWheelEvents.removeElement(userEvent);
					} else {
						mousePressedEvents.removeElement(userEvent);
						mouseReleasedEvents.removeElement(userEvent);
					}
				} else {
					keyPressedEvents.removeElement(userEvent);
					keyReleasedEvents.removeElement(userEvent);
				}
			} else {
				if (userEvent.isMouseActivated()) {
					if (userEvent.isMouseWheelEvent()) {
						mouseWheelEvents.removeElement(userEvent);
					} else {
						mousePressedEvents.removeElement(userEvent);
					}
				} else { 
					keyPressedEvents.removeElement(userEvent);
				}
			}
		}	
	}

	public void clearUserActions() {
		mousePressedEvents.clear();
		mouseReleasedEvents.clear();
		mouseWheelEvents.clear();
		keyPressedEvents.clear();
		keyReleasedEvents.clear();
	}
	
	/* (non-Javadoc)
	 * @see ca.uvic.csr.shrimp.DisplayBean.listener.ShrimpKeyListener#resetKeys()
	 */
	public void resetKeys() {
		isCtrlPressed = false;
		isShiftPressed = false;
		isAltPressed = false;
	}
		
}